@@ -21,10 +21,11 @@ use scale_info::{
     Registry,
     TypeInfo,
 };
+use syn::parse_quote;
 
 const MOD_PATH: &[&str] = &["subxt_codegen", "types", "tests"];
 
-fn get_mod<'a>(module: &'a Module, path_segs: &[&'static str]) -> Option<&'a Module<'a>> {
+fn get_mod<'a>(module: &'a Module, path_segs: &[&'static str]) -> Option<&'a Module> {
     let (mod_name, rest) = path_segs.split_first()?;
     let mod_ident = Ident::new(mod_name, Span::call_site());
     let module = module.children.get(&mod_ident)?;
@@ -64,7 +65,7 @@ fn generate_struct_with_primitives() {
             pub mod tests {
                 use super::root;
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct S {
                     pub a: ::core::primitive::bool,
                     pub b: ::core::primitive::u32,
@@ -110,12 +111,12 @@ fn generate_struct_with_a_struct_field() {
             pub mod tests {
                 use super::root;
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Child {
                     pub a: ::core::primitive::i32,
                 }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Parent {
                     pub a: ::core::primitive::bool,
                     pub b: root::subxt_codegen::types::tests::Child,
@@ -155,10 +156,10 @@ fn generate_tuple_struct() {
                 pub mod tests {
                     use super::root;
 
-                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                    #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                     pub struct Child(pub ::core::primitive::i32,);
 
-                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                    #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                     pub struct Parent(pub ::core::primitive::bool, pub root::subxt_codegen::types::tests::Child,);
                 }
             }
@@ -237,34 +238,34 @@ fn derive_compact_as_for_uint_wrapper_structs() {
             pub mod tests {
                 use super::root;
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Su128 { pub a: ::core::primitive::u128, }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Su16 { pub a: ::core::primitive::u16, }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Su32 { pub a: ::core::primitive::u32, }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Su64 { pub a: ::core::primitive::u64, }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Su8 { pub a: ::core::primitive::u8, }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct TSu128(pub ::core::primitive::u128,);
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct TSu16(pub ::core::primitive::u16,);
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct TSu32(pub ::core::primitive::u32,);
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct TSu64(pub ::core::primitive::u64,);
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct TSu8(pub ::core::primitive::u8,);
             }
         }
@@ -300,7 +301,7 @@ fn generate_enum() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub enum E {
                     # [codec (index = 0)]
                     A,
@@ -358,7 +359,7 @@ fn compact_fields() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub enum E {
                     # [codec (index = 0)]
                     A {
@@ -369,12 +370,12 @@ fn compact_fields() {
                     B( #[codec(compact)] ::core::primitive::u32,),
                 }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct S {
                     #[codec(compact)] pub a: ::core::primitive::u32,
                 }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct TupleStruct(#[codec(compact)] pub ::core::primitive::u32,);
             }
         }
@@ -408,7 +409,7 @@ fn generate_array_field() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct S {
                     pub a: [::core::primitive::u8; 32usize],
                 }
@@ -445,7 +446,7 @@ fn option_fields() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct S {
                     pub a: ::core::option::Option<::core::primitive::bool>,
                     pub b: ::core::option::Option<::core::primitive::u32>,
@@ -485,7 +486,7 @@ fn box_fields_struct() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct S {
                     pub a: ::std::boxed::Box<::core::primitive::bool>,
                     pub b: ::std::boxed::Box<::core::primitive::u32>,
@@ -525,7 +526,7 @@ fn box_fields_enum() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub enum E {
                     # [codec (index = 0)]
                     A(::std::boxed::Box<::core::primitive::bool>,),
@@ -565,7 +566,7 @@ fn range_fields() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct S {
                     pub a: ::core::ops::Range<::core::primitive::u32>,
                     pub b: ::core::ops::RangeInclusive<::core::primitive::u32>,
@@ -609,12 +610,12 @@ fn generics() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Bar {
                     pub b: root::subxt_codegen::types::tests::Foo<::core::primitive::u32>,
                     pub c: root::subxt_codegen::types::tests::Foo<::core::primitive::u8>,
                 }
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Foo<_0> {
                     pub a: _0,
                 }
@@ -657,12 +658,12 @@ fn generics_nested() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Bar<_0> {
                     pub b: root::subxt_codegen::types::tests::Foo<_0, ::core::primitive::u32>,
                 }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct Foo<_0, _1> {
                     pub a: _0,
                     pub b: ::core::option::Option<(_0, _1,)>,
@@ -708,7 +709,7 @@ fn generate_bitvec() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct S {
                     pub lsb: ::subxt::bitvec::vec::BitVec<::core::primitive::u8, root::bitvec::order::Lsb0>,
                     pub msb: ::subxt::bitvec::vec::BitVec<::core::primitive::u16, root::bitvec::order::Msb0>,
@@ -761,12 +762,12 @@ fn generics_with_alias_adds_phantom_data_marker() {
             quote! {
                 pub mod tests {
                     use super::root;
-                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
+                    #[derive(::subxt::codec::CompactAs, ::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                     pub struct NamedFields<_0> {
                         pub b: ::core::primitive::u32,
                         #[codec(skip)] pub __subxt_unused_type_params: ::core::marker::PhantomData<_0>
                     }
-                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                    #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                     pub struct UnnamedFields<_0, _1> (
                         pub (::core::primitive::u32, ::core::primitive::u32,),
                         #[codec(skip)] pub ::core::marker::PhantomData<(_0, _1)>
@@ -829,20 +830,20 @@ fn modules() {
                         pub mod b {
                             use super::root;
 
-                            #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                            #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                             pub struct Bar {
                                 pub a: root::subxt_codegen::types::tests::m::a::Foo,
                             }
                         }
 
-                        #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                        #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                         pub struct Foo;
                     }
 
                     pub mod c {
                         use super::root;
 
-                        #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                        #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                         pub struct Foo {
                             pub a: root::subxt_codegen::types::tests::m::a::b::Bar,
                         }
@@ -879,10 +880,112 @@ fn dont_force_struct_names_camel_case() {
             pub mod tests {
                 use super::root;
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug)]
                 pub struct AB;
             }
         }
         .to_string()
     )
 }
+
+#[test]
+fn apply_user_defined_derives_for_all_types() {
+    #[allow(unused)]
+    #[derive(TypeInfo)]
+    struct A(B);
+
+    #[allow(unused)]
+    #[derive(TypeInfo)]
+    struct B;
+
+    let mut registry = Registry::new();
+    registry.register_type(&meta_type::<A>());
+    let portable_types: PortableRegistry = registry.into();
+
+    // configure derives
+    let mut derives = DerivesRegistry::default();
+    derives.extend_for_all(vec![parse_quote!(Clone), parse_quote!(Eq)]);
+
+    let type_gen =
+        TypeGenerator::new(&portable_types, "root", Default::default(), derives);
+    let types = type_gen.generate_types_mod();
+    let tests_mod = get_mod(&types, MOD_PATH).unwrap();
+
+    assert_eq!(
+        tests_mod.into_token_stream().to_string(),
+        quote! {
+            pub mod tests {
+                use super::root;
+
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Clone, Debug, Eq)]
+                pub struct A(pub root :: subxt_codegen :: types :: tests :: B,);
+
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Clone, Debug, Eq)]
+                pub struct B;
+            }
+        }
+            .to_string()
+    )
+}
+
+#[test]
+fn apply_user_defined_derives_for_specific_types() {
+    #[allow(unused)]
+    #[derive(TypeInfo)]
+    struct A(B);
+
+    #[allow(unused)]
+    #[derive(TypeInfo)]
+    struct B(C);
+
+    #[allow(unused)]
+    #[derive(TypeInfo)]
+    struct C;
+
+    let mut registry = Registry::new();
+    registry.register_type(&meta_type::<A>());
+    let portable_types: PortableRegistry = registry.into();
+
+    // configure derives
+    let mut derives = DerivesRegistry::default();
+    // for all types
+    derives.extend_for_all(vec![parse_quote!(Eq)]);
+    // for specific types
+    derives.extend_for_type(
+        parse_quote!(subxt_codegen::types::tests::B),
+        vec![parse_quote!(Hash)],
+    );
+    // duplicates (in this case `Eq`) will be combined (i.e. a set union)
+    derives.extend_for_type(
+        parse_quote!(subxt_codegen::types::tests::C),
+        vec![
+            parse_quote!(Eq),
+            parse_quote!(Ord),
+            parse_quote!(PartialOrd),
+        ],
+    );
+
+    let type_gen =
+        TypeGenerator::new(&portable_types, "root", Default::default(), derives);
+    let types = type_gen.generate_types_mod();
+    let tests_mod = get_mod(&types, MOD_PATH).unwrap();
+
+    assert_eq!(
+        tests_mod.into_token_stream().to_string(),
+        quote! {
+            pub mod tests {
+                use super::root;
+
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug, Eq)]
+                pub struct A(pub root :: subxt_codegen :: types :: tests :: B,);
+
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug, Eq, Hash)]
+                pub struct B(pub root :: subxt_codegen :: types :: tests :: C,);
+
+                #[derive(::subxt::codec::Decode, ::subxt::codec::Encode, Debug, Eq, Ord, PartialOrd)]
+                pub struct C;
+            }
+        }
+            .to_string()
+    )
+}
